#lang racket

(require vigracket)
(require (rename-in 2htdp/image
                    (save-image save-plt-image)
                    (image-width plt-image-width)
                    (image-height plt-image-height)))

(define background_img (image->bitmap (load-image (path->string (build-path (current-directory) "Mensch.jpg")))))

(define start_x 49)
(define start_y 49)

(define schritt_x 70.4)
(define schritt_y 70.1)

(define anzahl_x 11)
(define anzahl_y 11)

(define (i->x i)
  (+ start_x (* i schritt_x)))

(define (j->y j)
  (+ start_y (* j schritt_y)))

;; Pos ist eine indizierte Position: i,j mit 0 < i,j < 11
(define (zeichne-punkt i j [img background_img] [rad 20])
  (if (and (< -1 i anzahl_x)  (< -1 j anzahl_y))  
    (underlay/xy img
                (- (i->x i) rad)
                (- (j->y j) rad)
                (circle rad 'solid 'red))
    (error "Es muss gelten: 0 < i,j < 11")))
    
(define start_gelb-pos    '((0 . 0) (1  . 0) (0 .  1) ( 1 .  1)))
(define start_grün-pos    '((9 . 0) (10 . 0) (9 .  1) (10 .  1)))
(define start_rot-pos     '((9 . 9) (10 . 9) (9 . 10) (10 . 10)))
(define start_schwarz-pos '((0 . 9) (1  . 9) (0 . 10) ( 1 . 10)))

(define haus_gelb-pos    '((1 . 5) (2 . 5) (3 . 5) (4 . 5)))
(define haus_grün-pos    '((5 . 1) (5 . 2) (5 . 3) (5 . 4)))
(define haus_rot-pos     '((6 . 5) (7 . 5) (8 . 5) (9 . 5)))
(define haus_schwarz-pos '((5 . 6) (5 . 7) (5 . 8) (5 . 9)))

(define gelb->grün-pos    '(( 0 .  4) (1 . 4) (2 . 4) (3 . 4) (4 . 4) (4 . 3) (4 . 2) (4 . 1) ( 4 .  0) ( 5 .  0)))
(define grün->rot-pos     '(( 6 .  0) (6 . 1) (6 . 2) (6 . 3) (6 . 4) (7 . 4) (8 . 4) (9 . 4) (10 .  4) (10 .  5)))
(define rot->schwarz-pos  '((10 .  6) (9 . 6) (8 . 6) (7 . 6) (6 . 6) (6 . 7) (6 . 8) (6 . 9) ( 6 . 10) ( 5 . 10)))
(define schwarz->gelb-pos '(( 4 . 10) (4 . 9) (4 . 8) (4 . 7) (4 . 6) (3 . 6) (2 . 6) (1 . 6) ( 0 .  6) ( 0 .  5)))

(define spiel-pos (append                                                              ;;Indizes:
                   start_gelb-pos start_grün-pos start_rot-pos start_schwarz-pos       ;; 0 - 15 Startfelder (4xGe, 4xGr, 4xR, 4xS)
                   haus_gelb-pos  haus_grün-pos haus_rot-pos haus_schwarz-pos          ;;16 - 31 Hausfelder  (4xGe, 4xGr, 4xR, 4xS)
                   gelb->grün-pos  grün->rot-pos rot->schwarz-pos schwarz->gelb-pos))  ;;32 - 71 Spielfelder (einmal rum, Start bei Gelb)
 
;;Start-Zustand
(define spiel-zustand0 (append '(yellow yellow yellow yellow green green green green red red red red  black black black black)
                               (make-list 56 'none)))   
;;End-Zustand
(define spiel-zustand1 (append (make-list 16 'none)
                               '(yellow yellow yellow yellow green green green green red red red red  black black black black)
                               (make-list 40 'none)))   
;;Echtes Spiel
(define spiel-zustand2 (append '(yellow yellow none none green green green none  red none none none  black black black none) ;Startfelder
                               '(yellow   none none none  none  none  none none none none none  red   none  none  none none) ;Hausfelder
                               '(none none none none none none none none none none)  ;Gelb->Grün
                               '(green none none yellow none none none none none none)  ;Grün->Rot
                               '(none none red none none none none none none none)  ;Rot->Schwarz
                               '(black none none none none red none none none none)  ;Schwarz->Gelb
                               ))       

(define (spielfigur farbe radius)
  (let ((kopfradius (* 2/3 radius)))
    (underlay/xy (triangle (* 2 radius) 'solid farbe)
                 (/ kopfradius 2) (- kopfradius)
                 (circle kopfradius 'solid farbe))))

(define (spielfigur_mit_rahmen farbe radius)
  (let ((rahmenfarbe (if (eq? farbe 'black) 'white 'black)))
    (underlay (spielfigur rahmenfarbe (* radius 11/10))
              (spielfigur farbe radius))))

(define (zeichne-zustand zustand [pos-liste spiel-pos] [img background_img] [groesse 20])
  (if (or (empty? zustand) (empty? pos-liste))
      img
      (if (eq? (car zustand) 'none)
          (zeichne-zustand (cdr zustand) (cdr pos-liste) img groesse)
          (let ((i (car (car pos-liste)))
                (j (cdr (car pos-liste))))   
            (underlay/xy
             (zeichne-zustand (cdr zustand) (cdr pos-liste) img groesse)
             (- (i->x i) groesse)
             (- (j->y j) groesse)
             (spielfigur_mit_rahmen (car zustand) groesse))))))

(zeichne-zustand spiel-zustand0)
(zeichne-zustand spiel-zustand1)
(zeichne-zustand spiel-zustand2)